<?php

namespace Drupal\countdown_scheduler\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides a 'Countdown scheduler component' block.
 *
 * @Block(
 *   id = "countdown_scheduler_block",
 *   admin_label = @Translation("Countdown scheduler Block"),
 * )
 */
class CountDownSchedulerBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'countdownscheduler_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $countdownDatetime = $config['countdown_date'] ?? NULL;
    $countdownTitle = $config['countdown_title'] ?? NULL;
    $countdownColor = $config['countdown_color'] ?? '#000';
    $countdownDesc = $config['countdown_desc'] ?? NULL;
    $countdownClosure = $config['countdown_closuremsg'] ?? NULL;
    $countdownMode = $config['countdown_disp_mode'] ?? 'light';

    if (!empty($countdownDatetime)) {
      return [
        '#theme' => 'countdown_block',
        '#timerdata' => $countdownDatetime,
        '#timertitle' => $countdownTitle,
        '#timercolor' => $countdownColor,
        '#timerdesc' => $countdownDesc,
        '#timerclosure' => $countdownClosure,
        '#timer_dispmode' => $countdownMode,
        '#attached' => [
          'library' => [
            'countdown_scheduler/timer-component',
          ],
          'drupalSettings' => [
            'countdown_scheduler' => [
              'timerdate' => $countdownDatetime,
            ],
          ],
        ],
      ];
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['schedulersection'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Scheduler section'),
    ];
    $form['schedulersection']['countdown_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Section Title'),
      '#required' => FALSE,
      '#placeholder' => $this->t('Enter title here'),
      '#default_value' => $config['countdown_title'] ?? NULL,
    ];
    $form['schedulersection']['countdown_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Count Down Date'),
      '#default_value' => isset($config['countdown_date']) ? new DrupalDateTime($config['countdown_date']) : NULL,
      '#date_date_element' => 'date',
      '#date_time_element' => 'time',
      '#date_year_range' => '0:+2',
      '#required' => TRUE,
      '#description' => $this->t('<i>Please select only future dates</i>'),
    ];
    $form['schedulersection']['countdown_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Count Down Color'),
      '#default_value' => $config['countdown_color'] ?? '#000',
      '#description' => $this->t('<i>Choose a color for the count down section using the color picker.</i>'),
    ];
    $form['schedulersection']['countdown_desc'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Description'),
      '#format' => $config['countdown_desc_format'] ?? 'basic_html',
      '#rows' => 5,
      '#default_value' => $config['countdown_desc'] ?? NULL,
      '#placeholder' => $this->t('Enter description'),
      '#description' => $this->t('<i>Please enter the section description.</i>'),
    ];
    $form['schedulersection']['countdown_disp_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Choose display mode'),
      '#options' => [
        'light' => $this->t('Light'),
        'dark' => $this->t('Dark'),
      ],
      '#default_value' => $config['countdown_disp_mode'] ?? 'light',
      '#description' => $this->t('<i>Please select Display mode. If Dark mode is selected please ensure to choose the timer section color with lighter colors. </i>'),
    ];
    $form['schedulersection']['countdown_closuremsg'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Countdown Closure Text'),
      '#required' => FALSE,
      '#placeholder' => $this->t('Enter closure text'),
      '#default_value' => $config['countdown_closuremsg'] ?? 'Countdown is Closed',
      '#description' => $this->t('<i>Please enter the count down closure text.</i>'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    if ($form_state->getValue(['schedulersection', 'countdown_date'])) {
      $convertedTime = strtotime($form_state->getValue(['schedulersection', 'countdown_date']));
      $currTime = time();
      if ($convertedTime < $currTime) {
        $form_state->setErrorByName('countdown_date', $this->t('Please enter future date.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    if ($form_state->getValue(['schedulersection', 'countdown_date'])) {
      $datetimer = $form_state->getValue(['schedulersection', 'countdown_date']);
      $this->configuration['countdown_date'] = $datetimer->format('Y-m-d H:i:s');
    }
    if ($form_state->getValue(['schedulersection', 'countdown_title'])) {
      $this->configuration['countdown_title'] = $form_state->getValue(['schedulersection', 'countdown_title']);
    }
    if ($form_state->getValue(['schedulersection', 'countdown_color'])) {
      $this->configuration['countdown_color'] = $form_state->getValue(['schedulersection', 'countdown_color']);
    }
    if ($form_state->getValue(['schedulersection', 'countdown_desc'])) {
      $desc = $form_state->getValue(['schedulersection', 'countdown_desc']);
      $this->configuration['countdown_desc'] = $desc['value'];
      $this->configuration['countdown_desc_format'] = $desc['format'] ?? 'basic_html';
    }
    if ($form_state->getValue(['schedulersection', 'countdown_closuremsg'])) {
      $this->configuration['countdown_closuremsg'] = $form_state->getValue(['schedulersection', 'countdown_closuremsg']);
    }
    if ($form_state->getValue(['schedulersection', 'countdown_disp_mode'])) {
      $this->configuration['countdown_disp_mode'] = $form_state->getValue(['schedulersection', 'countdown_disp_mode']);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'access countdown timer');
  }

}
